# Copyright (c) 2002 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id: commands.py,v 1.2 2002/12/24 14:07:46 zagy Exp $


# twisted imports
from twisted.internet import protocol, reactor, defer
from twisted.python import failure, log

# system imports
import cPickle, sys, os

from batchprocessor import error


class CommandRunner(protocol.ProcessProtocol):
    """Writes a command and gets its result."""

    def __init__(self, command, resultDeferred):
        self.command = command
        self.buffer = ""
        self.errBuffer = ""
        self.resultDeferred = resultDeferred
    
    def connectionMade(self):
        s = cPickle.dumps(self.command)
        self.transport.write(s)
        self.transport.closeStdin()

    def outReceived(self, data):
        self.buffer += data

    def errReceived(self, data):
        self.errBuffer += data
    
    def processEnded(self, reason):
        if self.errBuffer.strip():
            log.msg("Stderr output from batch process: " + self.errBuffer)
        
        if not self.buffer:
            self.resultDeferred.errback(ValueError("no data received from batch process (check batch logs for errors)"))
            return
        
        try:
            result = cPickle.loads(self.buffer)
            if isinstance(result, error.BatchError):
                self.resultDeferred.errback(RuntimeError(result))
            else:
                self.resultDeferred.callback(result)
        except cPickle.UnpicklingError, e:
            self.resultDeferred.errback(e)


import batchprocessor
runnerPath = os.path.join(os.path.dirname(batchprocessor.__file__), "runner.py")

def runCommand(command):
    """Run a BatchCommand in a subprocess, returning Deferred for result."""
    result = defer.Deferred()
    p = CommandRunner(command, result)
    reactor.spawnProcess(p, sys.executable, [sys.executable, runnerPath], env=os.environ)
    return result
