# Copyright (c) 2002 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id: processpool.py,v 1.2 2002/12/24 14:07:46 zagy Exp $


"""A pool of processes running commands."""

import commands


class ProcessPool:
    """Runs a number of processes that process commands.

    A ProcessPool is a consumer for a producer of commands. Any queueing
    of commands should be done at the producer.

    Currently we run one process for each command.
    """

    def __init__(self, maxsize, producer):
        self.maxsize = maxsize
        self.waiting = 0 # number of commands requested from producer
        self.running = 0 # number of running processes
        self.producer = producer
        self.fetchCommands()

    def fetchCommands(self):
        """Tell producer to produce more commands."""
        for i in range(self.maxsize - (self.running + self.waiting)):
            self.waiting += 1
            self.producer.produceCommand(self.runCommand)
    
    def runCommand(self, command):
        """Run command in processes, returning Deferred of result."""
        if not self.running < self.maxsize:
            raise RuntimeError, "can't run more commands at this time."
        self.running += 1
        self.waiting -=1
        d = commands.runCommand(command)
        d.addBoth(self._processDone)
        return d

    def _processDone(self, result):
        """Callback from command result."""
        assert self.running > 0
        self.running -= 1
        self.fetchCommands()
        return result
