# Copyright (c) 2002 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id: word.py,v 1.4 2003/01/03 16:51:24 zagy Exp $


"""Commands for batch processor."""

# system imports
import os
import xml.dom.minidom

from batchprocessor import interfaces


class Silva2Word:
    """Command for converting Silva XML to MS Word format."""

    __implements__ = interfaces.ICommand
    
    def __init__(self, xmltext, frontFilename, templateFilename, description):
        self.xmltext = xmltext
        self.frontFilename = frontFilename
        self.templateFilename = templateFilename
        self.description = description

    def getSize(self):
        return len(self.xmltext) / 1024.0
    
    def getType(self):
        return "silva2word"

    def run(self):
        if os.name != "nt":
            # useful for debugging on POSIX
            raise RuntimeError, "I am not a DOC"

        from silva2word import SilvaDOMHandler
        dom = xml.dom.minidom.parseString(self.xmltext)
        handler = SilvaDOMHandler.SilvaDOMHandler(dom, self.frontFilename, 
            self.templateFilename, 2)
        handler.run()
        return handler.getDOC()

    def getDescription(self):
        return self.description


class Word2Silva:
    """Command for converting Word file back into XML."""

    __implements__ = interfaces.ICommand
    
    def __init__(self, filename, description):
        self.filename = os.path.abspath(filename)
        self.description = description

    def getSize(self):
        return os.path.getsize(self.filename) / 1024.0
    
    def getType(self):
        return "word2silva"

    def run(self):
        if os.name != "nt":
            # useful for debugging on POSIX
            return "<xxx/>"
        from word2silva import convert
        c = convert.Converter(self.filename)
        c.run()
        xml = u'<?xml version="1.0" encoding="UTF-8" ?>'
        xml += c.getXML()
        
        return xml
    
    def getDescription(self):
        return self.description
