# Copyright (c) 2002 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id: image.py,v 1.2 2002/12/24 14:07:47 zagy Exp $


"""Support for <image> tag."""

# system imports
import urllib2, posixpath, tempfile, shutil

# docma imports
from silva2word.render import Renderable


class Image(Renderable):
    """A <image> tag renderer.

    Image tag description:
    <image src='url://foo/blah.gif' caption='Test Image' id='blah' />
    """

    def __init__(self, handler):
        url, self.caption, self.id = self.getArgs(handler)
        self.imagePath = self.fetchImage(url)
    
    def fetchImage(self, url):
        """Download image, storing it in self.resultPath.

        Returns path where image is, or None.
        """
        resultPath = tempfile.mktemp() + posixpath.splitext(url)[1]
        try:
            f = urllib2.urlopen(url)
        except urllib2.URLError, e:
            print "Couldn't download image %s" % url
            print e
            return None
        result = open(resultPath, "wb")
        shutil.copyfileobj(f, result)
        result.close()
        return resultPath

    def getArgs(self, handler):
        """Return tuple (url, caption, id)."""
        node, get = handler.currNode, handler.getNodeAttribute
        return get(node, "src"), get(node, "caption"), get(node, "id")

    def renderWord(self, handler):
        if not self.imagePath:
            return
        selection = handler.app.Selection
        p = selection.Paragraphs.Add()
        p.Alignment = 1 # center the paragraph
        handler.doc.InlineShapes.AddPicture(self.imagePath, 0, 1, p.Range)
        selection.Start = selection.End - 1
        selection.TypeText("\n" + self.caption + "\n")
        p = selection.Paragraphs.Add()
        p.Alignment = 0


def renderWord(handler):
    """Render <image> tag."""
    img = Image(handler)
    img.renderWord(handler)
