# Copyright (c) 2002 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id: silva_course.py,v 1.2 2002/12/24 14:07:47 zagy Exp $


"""Handler for <silva_course> record tags."""

from silva2word.render import *


COURSE_CONTENT_TITLE = 'Global Description of course content'
COURSE_GOALS_TITLE = 'Goal(s) of the course'


class ContentField(Renderable):
    """Content that has title and <doc></doc> content."""

    def __init__(self, field, title, style):
        self.field = field
        self.title = title
        self.style = style
    
    def renderWord(self, handler):
        # add title
        app = handler.app
        app.Selection.Style = handler.getStyle(self.style)
        app.Selection.TypeText(self.title + '\n')

        # render content
        # the way we do this is pretty icky - I think SilvaDOMHandler needs to
        # refactored so we don't have to do this.
        currNode = handler.currNode
        handler.currNode = currNode.getElementsByTagName(self.field)[0].getElementsByTagName('doc')[0]
        handler._walkSubNodes(handler._container_elements + ('table',))
        handler.currNode = currNode


class Course(Record):

    descriptions = (('teaching_staff'   , 'Teaching Staff'),
                    ('coordinator'      , 'Coordinator'),
                    ('consulting_hour'  , 'Consulting Hour'),
                    ('term'             , 'Term'),
                    ('load'             , 'Load'),
                    ('educational_form' , 'Educational Form'),
                    ('examination'      , 'Examination'),
                    ('literature'       , 'Literature'))

    def construct(self):
        # title table
        t = self.add(Table())
        t.addRow(
            Cell([FieldText("course_title")], style="Course Title Heading"),
            Cell([FieldText("code")], style="Course Code Heading"),
            )
        t.setWidths(.8, .2)
        t.setPostProcessWord(self._postTitle)
        
        # layout table
        t = self.add(Table())
        t.setPostProcessWord(self._postLayout)
        t.addRow(Cell([ContentField("content", COURSE_CONTENT_TITLE ,'Course Goal and Content Heading'),
                       ContentField("goal", COURSE_GOALS_TITLE, 'Course Goal and Content Heading')])
                 )

        # details table
        t2 = Table(subtable=1)
        t2.setWidths(.25, .75)
        t2.setPostProcessWord(self._postDetails)
        for fieldName, description in self.descriptions:
            t2.addRow(Cell([Text(description)], style="Course Details Heading"),
                      Cell([FieldText(fieldName)], style="Table Body Text"))
        t.addRow(Cell([t2]))
    
    def _postTitle(self, handler, table):
        """Post processing for title table."""
        table.Borders(3).Visible = 1
    
    def _postLayout(self, handler, table):
        """Post processing for layout table."""
        doc = handler.doc
        page_height = (doc.PageSetup.PageHeight - doc.PageSetup.TopMargin - 
                       doc.PageSetup.BottomMargin - 10 - 36)
        table.Cell(1, 1).Height = page_height*.65
        table.Cell(2, 1).Height = page_height*.35
        table.Cell(2, 1).VerticalAlignment = 3

    def _postDetails(self, handler, table):
        table.TopPadding = 0.8
        table.BottomPadding = 0.8


def renderWord(handler):
    """Render a 'silva_course' record."""
    c = Course()
    c.renderWord(handler)
